"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const debugLib = require("debug");
const path = require("path");
const utils_1 = require("./utils");
const debug = debugLib("azure-functions-pack:PackhostGenerator");
class PackhostGenerator {
    constructor(options) {
        this.functionsMap = new Map();
        this.options = options;
        this.options.indexFileName = this.options.indexFileName || "index.js";
        this.options.outputPath = this.options.outputPath || ".funcpack";
        debug("Created new PackhostGenerator for project at: %s", this.options.projectRootPath);
    }
    updateProject() {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Starting update of Project");
            yield this.throwIfInFunction();
            yield this.load();
            yield this.createOutputDirectory();
            yield this.createHostFile();
            yield this.updateFunctionJSONs();
            debug("Completed update of project");
        });
    }
    throwIfInFunction() {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Checking if we're in a function");
            if (yield utils_1.FileHelper.exists(path.resolve(this.options.projectRootPath, "function.json"))) {
                throw new Error("function.json detected: run this from "
                    + "the root of your Function App, not inside of a Function");
            }
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            const functions = (yield utils_1.FileHelper.readdir(this.options.projectRootPath))
                .filter((item) => __awaiter(this, void 0, void 0, function* () { return (yield utils_1.FileHelper.stat(path.resolve(this.options.projectRootPath, item))).isDirectory(); }));
            debug("Found these directories in project root: %s", functions.join(", "));
            for (const item of functions) {
                if (yield utils_1.FileHelper.exists(path.resolve(this.options.projectRootPath, item, "function.json"))) {
                    const fn = yield this.loadFunction(item);
                    if (fn !== null) {
                        this.functionsMap.set(item, fn);
                    }
                }
            }
        });
    }
    loadFunction(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let entryPoint = null;
            let scriptFile = null;
            let originalEntryPoint = false;
            let originalScriptFile = false;
            debug("Found function: %s", name);
            const fxJsonPath = path.resolve(this.options.projectRootPath, name, "function.json");
            const fxJson = yield utils_1.FileHelper.readFileAsJSON(fxJsonPath);
            if (fxJson._originalScriptFile) {
                debug("Found originalScriptFile setting: %s", fxJson._originalScriptFile);
                scriptFile = fxJson._originalScriptFile;
                originalScriptFile = fxJson._originalScriptFile;
            }
            else if (fxJson.scriptFile && fxJson.scriptFile.endsWith(".js") && !fxJson._originalScriptFile) {
                scriptFile = fxJson.scriptFile;
                originalScriptFile = fxJson.scriptFile;
            }
            else if (fxJson.scriptFile && !fxJson.scriptFile.endsWith(".js") && !fxJson._originalScriptFile) {
                return null;
            }
            else {
                let dir = yield utils_1.FileHelper.readdir(path.resolve(this.options.projectRootPath, name));
                dir = dir.filter((f) => f.endsWith(".js"));
                if (dir.length === 1) {
                    scriptFile = dir[0];
                }
                else if (dir.find((v, i, o) => {
                    return v === "index.js";
                })) {
                    scriptFile = "index.js";
                }
                else {
                    debug("Function %s does not have a valid start file", name, {
                        directory: dir,
                    });
                    return null;
                }
                originalScriptFile = scriptFile;
            }
            if (fxJson._originalEntryPoint) {
                debug("Found originalEntryPoint setting: %s", fxJson._originalEntryPoint);
                entryPoint = fxJson._originalEntryPoint;
                originalEntryPoint = fxJson._originalEntryPoint;
            }
            else if (fxJson.entryPoint && fxJson._originalEntryPoint !== false) {
                entryPoint = fxJson.entryPoint;
                originalEntryPoint = fxJson.entryPoint;
            }
            debug("Loaded function(%s) using entryPoint: %s - scriptFile: %s", name, scriptFile, entryPoint);
            return Promise.resolve({
                name,
                scriptFile,
                entryPoint,
                _originalEntryPoint: originalEntryPoint,
                _originalScriptFile: originalScriptFile,
            });
        });
    }
    createOutputDirectory() {
        return __awaiter(this, void 0, void 0, function* () {
            const outputDirPath = path.join(this.options.projectRootPath, this.options.outputPath);
            if (yield utils_1.FileHelper.exists(outputDirPath)) {
                debug("Deleting previous output directory: %s", this.options.outputPath);
                yield utils_1.FileHelper.rimraf(outputDirPath);
            }
            debug("Creating output directory: %s", outputDirPath);
            yield utils_1.FileHelper.mkdir(outputDirPath);
        });
    }
    createHostFile() {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Generating host file");
            const exportStrings = [];
            for (const [name, fx] of this.functionsMap) {
                const fxvar = this.safeFunctionName(fx.name);
                let exportStmt = `    "${fxvar}": require("../${fx.name}/${fx._originalScriptFile}")`;
                if (fx.entryPoint) {
                    exportStmt += `.${fx.entryPoint}`;
                }
                exportStrings.push(exportStmt);
            }
            let exportString = exportStrings.reduce((p, c, i, a) => p + c + ((i !== exportStrings.length - 1) ? ",\n" : "\n"), "");
            exportString = "module.exports = {\n" + exportString + "}";
            debug("Writing contents to host file");
            yield utils_1.FileHelper.writeFileUtf8(path.join(this.options.projectRootPath, this.options.outputPath, this.options.indexFileName), exportString);
        });
    }
    updateFunctionJSONs() {
        return __awaiter(this, void 0, void 0, function* () {
            debug("Updating Function JSONS");
            for (const [name, fx] of this.functionsMap) {
                debug("Updating function(%s)", name);
                const fxJsonPath = path.resolve(this.options.projectRootPath, name, "function.json");
                const fxvar = this.safeFunctionName(fx.name);
                const fxJson = yield utils_1.FileHelper.readFileAsJSON(fxJsonPath);
                fxJson._originalEntryPoint = fx._originalEntryPoint;
                fxJson._originalScriptFile = fx._originalScriptFile;
                fxJson.scriptFile = `../${this.options.outputPath}/${this.options.indexFileName}`;
                fxJson.entryPoint = fxvar;
                yield utils_1.FileHelper.overwriteFileUtf8(fxJsonPath, JSON.stringify(fxJson, null, " "));
            }
        });
    }
    safeFunctionName(name) {
        return name.replace("-", "$dash");
    }
}
exports.PackhostGenerator = PackhostGenerator;
//# sourceMappingURL=packhost-generator.js.map